twitter = {
    "Adjective": "형용사",
    "Adverb": "부사",
    "Alpha": "알파벳",
    "Conjunction": "접속사",
    "Determiner": "관형사",
    "Eomi": "어미",
    "Exclamation": "감탄사",
    "Foreign": "외국어, 한자 및 기타기호",
    "Hashtag": "트위터 해쉬태그",
    "Josa": "조사",
    "KoreanParticle": "(ex: ㅋㅋ)",
    "Noun": "명사",
    "Number": "숫자",
    "Modifier": "관형사",
    "PreEomi": "선어말어미",
    "Punctuation": "구두점",
    "ScreenName": "트위터 아이디",
    "Suffix": "접미사",
    "Unknown": "미등록어",
    "Verb": "동사",
    "Nic": "닉컴퍼니추가",
    "Name": "사람이름",
}